@@ -1,4 +1,4 @@
-/* Created at 2014-06-25-09-18-08*/
+/* Created at 2014-09-03-16-52-36*/
 #ifndef GETDNS_H
 #define GETDNS_H
 
@@ -183,6 +183,8 @@ typedef enum getdns_callback_type_t {
 #define GETDNS_RESPSTATUS_ALL_TIMEOUT_TEXT "All queries for the name timed out"
 #define GETDNS_RESPSTATUS_NO_SECURE_ANSWERS 903
 #define GETDNS_RESPSTATUS_NO_SECURE_ANSWERS_TEXT "The context setting for getting only secure responses was specified, and at least one DNS response was received, but no DNS response was determined to be secure through DNSSEC."
+#define GETDNS_RESPSTATUS_ALL_BOGUS_ANSWERS 904
+#define GETDNS_RESPSTATUS_ALL_BOGUS_ANSWERS_TEXT "The context setting for getting only secure responses was specified, and at least one DNS response was received, but all received responses for the requested name were bogus."
 
 /* Values Associated With Extensions */
 #define GETDNS_EXTENSION_TRUE 1000
@@ -256,6 +258,8 @@ typedef enum getdns_callback_type_t {
 #define GETDNS_RRTYPE_RKEY 57
 #define GETDNS_RRTYPE_TALINK 58
 #define GETDNS_RRTYPE_CDS 59
+#define GETDNS_RRTYPE_CDNSKEY 60
+#define GETDNS_RRTYPE_OPENPGPKEY 61
 #define GETDNS_RRTYPE_SPF 99
 #define GETDNS_RRTYPE_UINFO 100
 #define GETDNS_RRTYPE_UID 101
