@@ -725,16 +725,16 @@ set_ub_dns_transport(struct getdns_context* context,
     getdns_transport_t value) {
     switch (value) {
         case GETDNS_TRANSPORT_UDP_FIRST_AND_FALL_BACK_TO_TCP:
-            set_ub_string_opt(context, "do-udp", "yes");
-            set_ub_string_opt(context, "do-tcp", "yes");
+            set_ub_string_opt(context, "do-udp:", "yes");
+            set_ub_string_opt(context, "do-tcp:", "yes");
             break;
         case GETDNS_TRANSPORT_UDP_ONLY:
-            set_ub_string_opt(context, "do-udp", "yes");
-            set_ub_string_opt(context, "do-tcp", "no");
+            set_ub_string_opt(context, "do-udp:", "yes");
+            set_ub_string_opt(context, "do-tcp:", "no");
             break;
         case GETDNS_TRANSPORT_TCP_ONLY:
-            set_ub_string_opt(context, "do-udp", "no");
-            set_ub_string_opt(context, "do-tcp", "yes");
+            set_ub_string_opt(context, "do-udp:", "no");
+            set_ub_string_opt(context, "do-tcp:", "yes");
             break;
         default:
             /* TODO GETDNS_CONTEXT_TCP_ONLY_KEEP_CONNECTIONS_OPEN */
@@ -765,7 +765,7 @@ getdns_context_set_dns_transport(struct getdns_context *context,
 static void
 set_ub_limit_outstanding_queries(struct getdns_context* context, uint16_t value) {
     /* num-queries-per-thread */
-    set_ub_number_opt(context, "num-queries-per-thread", value);
+    set_ub_number_opt(context, "num-queries-per-thread:", value);
 }
 /*
  * getdns_context_set_limit_outstanding_queries
@@ -952,8 +952,8 @@ getdns_context_set_dnssec_trust_anchors(struct getdns_context *context,
 
 static void
 set_ub_dnssec_allowed_skew(struct getdns_context* context, uint32_t value) {
-    set_ub_number_opt(context, "val-sig-skew-min", value);
-    set_ub_number_opt(context, "val-sig-skew-max", value);
+    set_ub_number_opt(context, "val-sig-skew-min:", value);
+    set_ub_number_opt(context, "val-sig-skew-max:", value);
 }
 /*
  * getdns_context_set_dnssec_allowed_skew
@@ -1027,7 +1027,7 @@ static void
 set_ub_edns_maximum_udp_payload_size(struct getdns_context* context,
     uint16_t value) {
     /* max-udp-size */
-    set_ub_number_opt(context, "max-udp-size", value);
+    set_ub_number_opt(context, "max-udp-size:", value);
 }
 /*
  * getdns_context_set_edns_maximum_udp_payload_size
@@ -1814,7 +1814,7 @@ getdns_context_get_api_information(getdns_context* context) {
     }
     r = getdns_dict_util_set_string(result, "version_string", PACKAGE_VERSION);
     r |= getdns_dict_util_set_string(result, "implementation_string", PACKAGE_URL);
-    r |= getdns_dict_set_int(result, "resolver_type", context->resolution_type);
+    r |= getdns_dict_set_int(result, "resolution_type", context->resolution_type);
     settings = priv_get_context_settings(context);
     r |= getdns_dict_set_dict(result, "all_context", settings);
     getdns_dict_destroy(settings);
